package top.xinsin.controller;


import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import jakarta.websocket.*;
import jakarta.websocket.server.PathParam;
import jakarta.websocket.server.ServerEndpoint;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import top.xinsin.service.WebSocketService;
import top.xinsin.util.R;

import java.io.IOException;
import java.net.http.WebSocket;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;

@Component
@ServerEndpoint(value = "/api/auth/websocket/{userId}")
@Slf4j
public class WebSocketController {
    private static ConcurrentHashMap<String, WebSocketController> webSocketMap = new ConcurrentHashMap<>();

    private Session session;
    private String userId;
    public static WebSocketService webSocketService;

    @OnOpen
    public void onOpen(Session session, @PathParam("userId") String userId) {
        this.session = session;
        this.userId = userId;
        //加入map
        webSocketMap.put(userId, this);
        R<JSONObject> stringR = webSocketService.addVerify(userId,this);
        sendMessageByUserId(userId, stringR);
    }
    @OnClose
    public void onClose() {
        //从map中删除
        webSocketMap.remove(userId);
    }
    @OnMessage
    public void onMessage(String message, Session session) {
        sendMessageByUserId(userId,webSocketService.parseMessage(userId,message,webSocketMap));
    }
    @OnError
    public void onError(Session session, Throwable error) {
        log.error("用户错误:" + this.userId + ",原因:" + error.getMessage());
        error.printStackTrace();
    }
    /**
     * 向客户端发送消息
     */
    @SneakyThrows
    public void sendMessage(String message) {
        this.session.getBasicRemote().sendText(message);
    }

    /**
     * 通过userId向客户端发送消息
     */
    @SneakyThrows
    public void sendMessageByUserId(String userId, R<?> message) {
        if(webSocketMap.containsKey(userId)){
            webSocketMap.get(userId).sendMessage(JSON.toJSONString(message));
        }else{
            log.error("用户{}不在线",userId);
        }
    }
    @Scheduled(fixedRate = 1000)
    public void verifyTime(){
        List<String> expireTimes = webSocketService.getExpireTimes(webSocketMap.keys());
        expireTimes.forEach(e -> {
            if(webSocketMap.containsKey(e)){
                webSocketMap.get(e).sendMessage(JSON.toJSONString(R.success(new JSONObject().fluentPut("info","当前登录用户已过期"))));
            }
        });
    }
}
