package top.xinsin.service;

import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;
import top.xinsin.mapper.*;
import top.xinsin.pojo.*;
import top.xinsin.pojo.vo.AccountAndStoreAdmin;
import top.xinsin.util.HttpCodes;
import top.xinsin.util.R;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * @author xinsin
 * Created On 2023/4/24 21:11
 * @version 1.0
 */
@Service
@Slf4j
public class AccountService implements UserDetailsService {
    @Autowired
    private AccountMapper accountMapper;
    @Autowired
    private AuthRolesPermissionsMapper authRolesPermissionsMapper;
    @Autowired
    private AuthRolesMapper authRolesMapper;
    @Autowired
    private AuthPermissionsMapper authPermissionsMapper;
    @Autowired
    private VerifyMapper verifyMapper;
    @Resource
    private RedisTemplate<String, Object> redisTemplate;

    BCryptPasswordEncoder encoder = new BCryptPasswordEncoder();

    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        if (username == null) {
            throw new UsernameNotFoundException("用户名不能为空");
        } else {
            AuthAccount authAccount = this.accountMapper.selectOne(new LambdaQueryWrapper<AuthAccount>()
                    .eq(AuthAccount::getUsername, username)
                    .or()
                    .eq(AuthAccount::getEmail, username)
                    .eq(AuthAccount::getStatus, true));
            if (authAccount == null) {
                throw new UsernameNotFoundException("用户名或密码错误");
            } else {
                if (authAccount.getAuth().equals("商户")){
                    Verify verify = verifyMapper.selectOne(new LambdaQueryWrapper<Verify>().eq(Verify::getStoreId, authAccount.getId()));
                    if ("false".equals(verify.getVerifyResult())){
                        log.info("无法登录,因为商户管理员已拒绝");
                        return User.withUsername(authAccount.getUsername()).password("").build();
                    }
                }
                return User
                        .withUsername(authAccount.getUsername())
                        .password(authAccount.getPassword())
                        .roles("id=" + authAccount.getId(),"roleId=" + authAccount.getRoleId(),"auth=" + authAccount.getAuth())
                        .build();
            }
        }
    }

    public R<String> register(AuthAccount authAccount) {
        authAccount.setCreateTime(new Date());
        authAccount.setStatus(true);
        authAccount.setPassword(this.encoder.encode(authAccount.getPassword()));
        int insert = this.accountMapper.insert(authAccount);
        return insert == 1 ? R.success("注册成功") : R.failed(HttpCodes.HTTP_CODES555, "注册失败,您的用户名或者邮箱已经被注册过了");
    }

    public R<String> changeAccount(AuthAccount authAccount) {
        if (authAccount.getPassword() != null) {
            authAccount.setPassword(this.encoder.encode(authAccount.getPassword()));
        }

        int update = this.accountMapper.updateById(authAccount);
        return update == 1 ? R.success("修改用户信息成功") : R.failed(HttpCodes.HTTP_CODES555, "修改用户信息失败,请联系服务器管理员");
    }

    public R<String> changeStatus(Integer userId, Boolean status) {
        AuthAccount authAccount = new AuthAccount();
        authAccount.setId(userId);
        authAccount.setStatus(status);
        int i = this.accountMapper.updateById(authAccount);
        if (i == 1) {
            return status ? R.success("启用该用户成功") : R.success("禁用该用户成功");
        } else {
            return R.failed(HttpCodes.HTTP_CODES555, "修改用户状态失败");
        }
    }

    public R<String> changePassword(Integer userId, String oldPassword, String newPassword) {
        AuthAccount authAccount = new AuthAccount();
        authAccount.setPassword(this.encoder.encode(newPassword));
        LambdaQueryWrapper<AuthAccount> accountLambdaQueryWrapper = new LambdaQueryWrapper<>();
        accountLambdaQueryWrapper
                .eq(AuthAccount::getId, userId)
                .eq(AuthAccount::getPassword, this.encoder.encode(oldPassword));
        int update = this.accountMapper.update(authAccount, accountLambdaQueryWrapper);
        return update == 1 ? R.success("修改密码成功") : R.failed(HttpCodes.HTTP_CODES555, "修改密码失败,请联系服务器管理员");
    }

    public R<JSONObject> getUser(Integer page, Integer num) {
        Page<AuthAccount> accountPage = new Page<>(page, num);
        String auth = ((User) SecurityContextHolder
                .getContext()
                .getAuthentication()
                .getPrincipal())
                .getAuthorities()
                .stream()
                .filter(e -> e.getAuthority().startsWith("ROLE_auth="))
                .findFirst()
                .get()
                .getAuthority().replace("ROLE_auth=","");
        LambdaQueryWrapper<AuthAccount> userLambdaQueryWrapper = new LambdaQueryWrapper<>();
        if (auth.equals("租户管理员")){
            userLambdaQueryWrapper.in(AuthAccount::getAuth,"商户管理员","商户");
        }
        if (auth.equals("商户管理员")){
            userLambdaQueryWrapper.in(AuthAccount::getAuth,"商户");
        }
        System.out.println(auth);
        Page<AuthAccount> accountPage1 = this.accountMapper.selectPage(accountPage, userLambdaQueryWrapper);
        JSONObject jsonObject = new JSONObject();
        List<AuthAccount> collect = accountPage1.getRecords().stream().peek((e) -> e.setPassword(null)).toList();
        jsonObject
                .fluentPut("info", collect)
                .fluentPut("total", accountPage1.getTotal());
        return R.success(jsonObject);
    }

    public R<String> delUser(Integer id) {
        int delete = this.accountMapper.deleteById(id);
        return delete == 1 ? R.success("删除用户成功") : R.failed(HttpCodes.HTTP_CODES555, "删除用户失败,请联系服务器管理员");
    }

    public R<JSONObject> getUserInfo(int rolesId) {
        JSONObject globalReturnData = new JSONObject();
        JSONArray globalReturnPermissionList = new JSONArray();
        List<AuthRolesPermissions> authRolesPermissions = authRolesPermissionsMapper
                .selectList(new LambdaQueryWrapper<AuthRolesPermissions>()
                        .eq(AuthRolesPermissions::getRoleId, rolesId));
        List<Integer> permissionsList = authRolesPermissions.stream().map(AuthRolesPermissions::getPermissionId).toList();
        List<AuthPermissions> authPermissions = authPermissionsMapper.selectList(new LambdaQueryWrapper<AuthPermissions>()
                .in(AuthPermissions::getId, permissionsList));
        authPermissions.forEach(e -> {
            HashMap<String, String> returnPermissionJson = new HashMap<>();
            returnPermissionJson.put("permissionName",e.getPermissionName());
            returnPermissionJson.put("permissionCode",e.getPermissionCode());
            globalReturnPermissionList.add(returnPermissionJson);
        });
        AuthRoles authRoles = authRolesMapper.selectOne(new LambdaQueryWrapper<AuthRoles>().eq(AuthRoles::getId, rolesId));
//                角色名
        String roleName = authRoles.getRoleName();
        globalReturnData
                .fluentPut("info",globalReturnPermissionList)
                .fluentPut("roleName",roleName);
        return R.success(globalReturnData);
    }

    public R<JSONObject> addStoreAdmin(AccountAndStoreAdmin accountAndStoreAdmin) {
        AuthAccount authAccount = new AuthAccount(accountAndStoreAdmin.getUsername(), accountAndStoreAdmin.getAuth(), accountAndStoreAdmin.getEmail(), accountAndStoreAdmin.getRelName(), this.encoder.encode(accountAndStoreAdmin.getPassword()), accountAndStoreAdmin.getStatus(), accountAndStoreAdmin.getRoleId(), new Date());
        int insert = accountMapper.insert(authAccount);
        if (insert == 1){
            Verify verify = new Verify(accountAndStoreAdmin.getCreateId(), authAccount.getId(), accountAndStoreAdmin.getCountdown());
            int insert1 = verifyMapper.insert(verify);
            if (insert1 == 1){
//                redisTemplate.opsForValue().set(String.valueOf(authAccount.getId()),authAccount.getUsername(),verify.getCountdown(), TimeUnit.SECONDS);
                return R.success(new JSONObject().fluentPut("status","添加该商户成功").fluentPut("nextVerificationTime",verify.getCountdown()));
            }
        }
        return R.failed(HttpCodes.HTTP_CODES500,new JSONObject().fluentPut("status","添加商户失败"));
    }
}
