package top.xinsin.service;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.User;
import org.springframework.stereotype.Service;
import top.xinsin.mapper.AccountMapper;
import top.xinsin.mapper.AuthPermissionsMapper;
import top.xinsin.pojo.AuthAccount;
import top.xinsin.pojo.AuthPermissions;
import top.xinsin.pojo.AuthRoles;
import top.xinsin.pojo.vo.AuthPermissionsAndUserVo;
import top.xinsin.util.HttpCodes;
import top.xinsin.util.R;

import java.util.*;

import static org.springframework.data.util.Pair.toMap;

/**
 * @author xinsin
 * Created On 2023/4/24 21:13
 * @version 1.0
 */
@Service
public class AuthPermissionsService {

    @Autowired
    private AuthPermissionsMapper authPermissionsMapper;
    @Autowired
    private AccountMapper accountMapper;
    public R<Boolean> addAuthPermissions(AuthPermissions authPermissions) {
        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        authentication.getAuthorities().forEach(e -> {
            String authority = e.getAuthority();
            if (authority.startsWith("ROLE_id=")){
               authPermissions.setCreateId(Integer.parseInt(authority.replace("ROLE_id=", "")));
            }
        });
        authPermissions.setCreateTime(new Date());
        int insert = authPermissionsMapper.insert(authPermissions);
        return insert == 1 ? R.success(true) : R.failed(HttpCodes.HTTP_CODES555, false);
    }

    public R<Boolean> changeAuthPermissions(AuthPermissions authPermissions) {
        authPermissions.setUpdateTime(new Date());
        int update = authPermissionsMapper.updateById(authPermissions);
        return update == 1 ? R.success(true) : R.failed(HttpCodes.HTTP_CODES555, false);
    }

    public R<Boolean> delAuthPermissions(Integer id) {
        int delete = authPermissionsMapper.deleteById(id);
        return delete == 1 ? R.success(true) : R.failed(HttpCodes.HTTP_CODES555, false);
    }

    public R<JSONObject> getAuthPermissions(Integer page, Integer num) {
        Page<AuthPermissions> authPermissionsPage = new Page<>(page, num);
        Page<AuthPermissions> authPermissionsPage1 = authPermissionsMapper.selectPage(authPermissionsPage, null);
        List<AuthPermissions> records = authPermissionsPage1.getRecords();
        List<Integer> integers = records.stream().map(AuthPermissions::getCreateId).toList();
        List<AuthAccount> authAccounts = accountMapper.selectList(new LambdaQueryWrapper<AuthAccount>()
                .in(AuthAccount::getId, integers));
        Map<Integer,String > map = new HashMap<>();
        authAccounts.forEach(e -> map.put(e.getId(), e.getUsername()));
        List<AuthPermissionsAndUserVo> authPermissionsAndUserVos = records.stream().map(AuthPermissionsAndUserVo::new).toList();
        authPermissionsAndUserVos.forEach(e -> e.setCreateName(map.get(e.getCreateId())));
        return R.success(new JSONObject().fluentPut("info",authPermissionsAndUserVos).fluentPut("total",authPermissionsPage.getTotal()));
    }

    public R<String> changeAuthPermissionsStats(Integer authPermissionId, Boolean status) {
        return R.success("");
    }

    public R<List<JSONObject>> getAuthPermissionsId() {
        List<AuthPermissions> authRoles = authPermissionsMapper.selectList(null);
        ArrayList<JSONObject> jsonObjects = new ArrayList<>();
        authRoles.forEach(e -> {
            jsonObjects.add(new JSONObject().fluentPut("id",e.getId()).fluentPut("permissionName",e.getPermissionName()));
        });
        return R.success(jsonObjects);
    }
}
