package top.xinsin.service;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import top.xinsin.mapper.AccountMapper;
import top.xinsin.mapper.AuthPermissionsMapper;
import top.xinsin.mapper.AuthRolesMapper;
import top.xinsin.mapper.AuthRolesPermissionsMapper;
import top.xinsin.pojo.AuthPermissions;
import top.xinsin.pojo.AuthRoles;
import top.xinsin.pojo.AuthRolesPermissions;
import top.xinsin.util.HttpCodes;
import top.xinsin.util.R;

import java.util.*;

/**
 * @author xinsin
 * Created On 2023/4/29 20:28
 * @version 1.0
 */
@Service
public class AuthRolesPermissionsService {
    @Autowired
    private AuthRolesPermissionsMapper authRolesPermissionsMapper;
    @Autowired
    private AuthRolesMapper authRolesMapper;
    @Autowired
    private AuthPermissionsMapper authPermissionsMapper;
    @Autowired
    private AccountMapper accountMapper;
    public R<Boolean> addAuthRolesPermissions(AuthRolesPermissions authPermissions) {
        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        authentication.getAuthorities().forEach(e -> {
            String authority = e.getAuthority();
            if (authority.startsWith("ROLE_id=")){
                authPermissions.setCreateId(Integer.parseInt(authority.replace("ROLE_id=", "")));
            }
        });
        authPermissions.setCreateTime(new Date());
        int insert = authRolesPermissionsMapper.insert(authPermissions);
        return insert == 1 ? R.success(true) : R.failed(HttpCodes.HTTP_CODES555, false);
    }

    public R<Boolean> changeAuthRolesPermissions(AuthRolesPermissions authPermissions) {
        authPermissions.setUpdateTime(new Date());
        int update = authRolesPermissionsMapper.updateById(authPermissions);
        return update == 1 ? R.success(true) : R.failed(HttpCodes.HTTP_CODES555, false);
    }

    public R<Boolean> delAuthRolesPermissions(Integer roleId, Integer permissionId) {
        int delete = authRolesPermissionsMapper.delete(new LambdaQueryWrapper<AuthRolesPermissions>()
                .eq(AuthRolesPermissions::getRoleId,roleId)
                .eq(AuthRolesPermissions::getPermissionId,permissionId));
        return delete == 1 ? R.success(true) : R.failed(HttpCodes.HTTP_CODES555, false);
    }

    public R<JSONObject> getAuthRolesPermissions(Integer page, Integer num) {
        Page<AuthRoles> authRolesPage = new Page<>(page, num);
        Page<AuthRoles> authRolesPage1 = authRolesMapper.selectPage(authRolesPage, null);
        ArrayList<JSONObject> jsonObjects1 = new ArrayList<>();
        authRolesPage1.getRecords().forEach(e -> {
            JSONObject jsonObject = new JSONObject();
            jsonObject.fluentPut("roleName",e.getRoleName());
            LambdaQueryWrapper<AuthRolesPermissions> authRolesPermissionsLambdaQueryWrapper = new LambdaQueryWrapper<>();
            authRolesPermissionsLambdaQueryWrapper.eq(AuthRolesPermissions::getRoleId,e.getId());
            List<AuthRolesPermissions> authRolesPermissions = authRolesPermissionsMapper.selectList(authRolesPermissionsLambdaQueryWrapper);
            ArrayList<JSONObject> jsonObjects = new ArrayList<>();
            authRolesPermissions.forEach(a -> {
                JSONObject jsonObject1 = new JSONObject();
                LambdaQueryWrapper<AuthPermissions> authPermissionsLambdaQueryWrapper = new LambdaQueryWrapper<>();
                authPermissionsLambdaQueryWrapper.eq(AuthPermissions::getId,a.getPermissionId());
                AuthPermissions authPermissions = authPermissionsMapper.selectOne(authPermissionsLambdaQueryWrapper);
                jsonObject1.fluentPut("permissionName",authPermissions.getPermissionName()).fluentPut("permissionCode",authPermissions.getPermissionCode());
                jsonObjects.add(jsonObject1);
            });
            jsonObject.fluentPut("permissions",jsonObjects);
            jsonObjects1.add(jsonObject);
        });
        return R.success(new JSONObject().fluentPut("info",jsonObjects1).fluentPut("total",authRolesPage.getTotal()));
    }

    public R<String> changeAuthRolesPermissionsStats(Integer authPermissionId, Boolean status) {
        return R.success("");
    }

//    public R<List<JSONObject>> getAuthRolesPermissions() {
//        List<AuthRolesPermissions> authRoles = authRolesPermissionsMapper.selectList(null);
//        ArrayList<JSONObject> jsonObjects = new ArrayList<>();
//        authRoles.forEach(e -> {
//            jsonObjects.add(new JSONObject().fluentPut("id",e.getId()).fluentPut("permissionName",e.getPermissionName()));
//        });
//        return R.success(jsonObjects);
//    }
}
