package top.xinsin.service;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import top.xinsin.mapper.AccountMapper;
import top.xinsin.mapper.AuthPermissionsMapper;
import top.xinsin.mapper.AuthRolesMapper;
import top.xinsin.mapper.AuthRolesPermissionsMapper;
import top.xinsin.pojo.AuthAccount;
import top.xinsin.pojo.AuthPermissions;
import top.xinsin.pojo.AuthRoles;
import top.xinsin.pojo.AuthRolesPermissions;
import top.xinsin.pojo.vo.AuthPermissionsAndUserVo;
import top.xinsin.util.HttpCodes;
import top.xinsin.util.R;

import java.util.*;

/**
 * @author xinsin
 * Created On 2023/4/24 21:13
 * @version 1.0
 */
@Service
public class AuthRolesService {

    @Autowired
    private AuthRolesMapper authRolesMapper;
    @Autowired
    private AuthPermissionsMapper authPermissionsMapper;
    @Autowired
    private AuthRolesPermissionsMapper authRolesPermissionsMapper;
    public R<Boolean> addAuthRoles(AuthRoles authPermissions) {
        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        authentication.getAuthorities().forEach(e -> {
            String authority = e.getAuthority();
            if (authority.startsWith("ROLE_id=")){
                authPermissions.setCreateId(Integer.parseInt(authority.replace("ROLE_id=", "")));
            }
        });
        authPermissions.setCreateTime(new Date());
        int insert = authRolesMapper.insert(authPermissions);
        return insert == 1 ? R.success(true) : R.failed(HttpCodes.HTTP_CODES555, false);
    }

    public R<Boolean> changeAuthRoles(AuthRoles authPermissions) {
        authPermissions.setUpdateTime(new Date());
        int update = authRolesMapper.updateById(authPermissions);
        return update == 1 ? R.success(true) : R.failed(HttpCodes.HTTP_CODES555, false);
    }

    public R<Boolean> delAuthRoles(Integer id) {
        int delete = authRolesMapper.deleteById(id);
        return delete == 1 ? R.success(true) : R.failed(HttpCodes.HTTP_CODES555, false);
    }

    public R<JSONObject> getAuthRoles(Integer page, Integer num) {
        Page<AuthRoles> authPermissionsPage = new Page<>(page, num);
        Page<AuthRoles> authRolesPage = authRolesMapper.selectPage(authPermissionsPage, null);
        return R.success(new JSONObject().fluentPut("info",authRolesPage.getRecords()).fluentPut("total",authPermissionsPage.getTotal()));
    }

    public R<String> changeAuthRolesStats(Integer authPermissionId, Boolean status) {
        return R.success("");
    }

    public R<List<JSONObject>> getAuthRolesId() {
        List<AuthRoles> authRoles = authRolesMapper.selectList(null);
        ArrayList<JSONObject> jsonObjects = new ArrayList<>();
        authRoles.forEach(e -> {
            jsonObjects.add(new JSONObject().fluentPut("id",e.getId()).fluentPut("roleName",e.getRoleName()));
        });
        return R.success(jsonObjects);
    }

    public R<List<JSONObject>> getAuthPermissionId(Integer roleId) {
        LambdaQueryWrapper<AuthRolesPermissions> authRolesPermissionsLambdaQueryWrapper = new LambdaQueryWrapper<>();
        authRolesPermissionsLambdaQueryWrapper.eq(AuthRolesPermissions::getRoleId,roleId);
        List<AuthRolesPermissions> authRolesPermissions = authRolesPermissionsMapper.selectList(authRolesPermissionsLambdaQueryWrapper);
        ArrayList<JSONObject> jsonObjects = new ArrayList<>();
        authRolesPermissions.forEach(e ->{
            JSONObject jsonObject = new JSONObject();
            AuthPermissions authPermissions = authPermissionsMapper.selectOne(new LambdaQueryWrapper<AuthPermissions>().eq(AuthPermissions::getId, e.getPermissionId()));
            jsonObject
                    .fluentPut("name",authPermissions.getPermissionName())
                    .fluentPut("id",authPermissions.getId())
                    .fluentPut("description",authPermissions.getDescription())
                    .fluentPut("status",authPermissions.getStatus())
                    .fluentPut("code",authPermissions.getPermissionCode());
            jsonObjects.add(jsonObject);
        });
        return R.success(jsonObjects);
    }
}
