package top.xinsin.service;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import top.xinsin.mapper.CommissionMapper;
import top.xinsin.pojo.Commission;
import top.xinsin.util.HttpCodes;
import top.xinsin.util.R;

import java.util.Date;

/**
 * <p>
 *  服务类
 * </p>
 *
 * @author xinsin
 * @since 2023-04-24
 */
@Service
public class CommissionService {

    @Autowired
    private CommissionMapper commissionMapper;

    public R<Boolean> addCommission(Commission commission) {
        commission.setCreateTime(new Date());
        int insert = commissionMapper.insert(commission);
        return insert != 0 ? R.success(true) : R.failed(HttpCodes.HTTP_CODES555,false);
    }

    public R<Boolean> changeCommission(Commission commission) {
        commission.setUpdateTime(new Date());
        int update = commissionMapper.updateById(commission);
        return update != 0 ? R.success(true) : R.failed(HttpCodes.HTTP_CODES555,false);
    }

    public R<Boolean> delCommission(Integer id) {
        int delete = commissionMapper.deleteById(id);
        return delete != 0 ? R.success(true) : R.failed(HttpCodes.HTTP_CODES555,false);
    }

    public R<JSONObject> getCommission(Integer page, Integer num) {
        JSONObject jsonObject = new JSONObject();
        Page<Commission> commissionPage = new Page<>(page,num);
        Page<Commission> commissionPage1 = commissionMapper.selectPage(commissionPage, null);
        jsonObject.fluentPut("info",commissionPage1.getRecords()).fluentPut("total",commissionPage1.getTotal());
        return R.success(jsonObject);

    }

    public R<Double> getCommissionByPrice(Integer shopId, Double range) {
        LambdaQueryWrapper<Commission> commissionLambdaQueryWrapper = new LambdaQueryWrapper<>();
        commissionLambdaQueryWrapper
                .le(Commission::getPriceMax,range)
                .ge(Commission::getPriceMin,range)
                .eq(Commission::getStoreId,shopId);
        Commission commission = commissionMapper.selectOne(commissionLambdaQueryWrapper);
        if (commission != null){
            return R.success(commission.getCommission());
        }else{
            return R.failed(HttpCodes.HTTP_CODES555,0.0);
        }
    }

    public R<String> changeCommissionStatus(Integer commissionId, Boolean status) {
        Commission commission = new Commission();
        commission.setId(commissionId);
        commission.setStatus(status);
        int i = commissionMapper.updateById(commission);
        if (i == 1) {
            return status ? R.success("启用该佣金项成功") : R.success("禁用该佣金项成功");
        } else {
            return R.failed(HttpCodes.HTTP_CODES555, "修改佣金状态失败");
        }
    }
}
