package top.xinsin.service;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.read.listener.PageReadListener;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import jakarta.servlet.http.HttpServletResponse;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;
import top.xinsin.enums.OrderStatus;
import top.xinsin.enums.SearchType;
import top.xinsin.mapper.CommissionMapper;
import top.xinsin.mapper.OrderMapper;
import top.xinsin.mapper.StoreMapper;
import top.xinsin.pojo.Commission;
import top.xinsin.pojo.Order;
import top.xinsin.pojo.OrderByExcel;
import top.xinsin.pojo.Store;
import top.xinsin.util.HttpCodes;
import top.xinsin.util.R;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * @author xinsin
 * Created On 2023/4/24 21:12
 * @version 1.0
 */
@Service
@Slf4j
public class OrderService extends ServiceImpl<OrderMapper,Order> implements OrderServiceInter {
    @Autowired
    private StoreMapper storeMapper;
    @Autowired
    private CommissionMapper commissionMapper;

    public R<String> addOrder(Order order) {
        order.setOrderTime(new Date());
        boolean insert = this.save(order);
        return insert ? R.success("订单添加成功") : R.failed(HttpCodes.HTTP_CODES555, "订单添加失败,请联系服务器管理员");
    }

    public R<JSONObject> getOrder(Integer page, Integer num) {
        Page<Order> orderPage = new Page<>((long)page, (long)num);
        Page<Order> orderPage1 = this.page(orderPage, null);
        orderPage1.getRecords().forEach(e -> {
            LambdaQueryWrapper<Store> orderVOLambdaQueryWrapper = new LambdaQueryWrapper<>();
            orderVOLambdaQueryWrapper.eq(Store::getId,e.getStoreId());
            Store store = storeMapper.selectOne(orderVOLambdaQueryWrapper);
            e.setStoreName(store.getName());

            LambdaQueryWrapper<Commission> commissionLambdaQueryWrapper = new LambdaQueryWrapper<>();
            commissionLambdaQueryWrapper
                    .eq(Commission::getStoreId,e.getStoreId())
                    .le(Commission::getPriceMax,e.getCommission())
                    .ge(Commission::getPriceMin,e.getCommission());
            Commission commission = commissionMapper.selectOne(commissionLambdaQueryWrapper);
            if (commission != null){
                e.setCommission(commission.getCommission());
            }else{
                e.setCommission(0.0);
            }
        });
        JSONObject jsonObject = new JSONObject();
        jsonObject.fluentPut("info", orderPage1.getRecords()).fluentPut("total", orderPage1.getTotal());
        return R.success(jsonObject);
    }

    public R<Boolean> getOrderUserInfo(String content) {
        LambdaQueryWrapper<Order> orderLambdaQueryWrapper = new LambdaQueryWrapper<>();
        orderLambdaQueryWrapper.eq(Order::getWechatNo, content).or().eq(Order::getWangNo, content).or().eq(Order::getAlipayNo, content).or().eq(Order::getPhoneNumber, content).or().eq(Order::getAlipayName, content);
        List<Order> orders = this.list(orderLambdaQueryWrapper);
        return orders.size() == 0 ? R.success(false) : R.success(true);
    }

    public R<JSONObject> searchUserOrder(SearchType searchType, String searchContent, Boolean batch, Integer page, Integer num) {
        LambdaQueryWrapper<Order> orderLambdaQueryWrapper = new LambdaQueryWrapper<>();
        if (batch) {
            String[] split = searchContent.split(",");
            String[] var8 = split;
            int var9 = split.length;

            for(int var10 = 0; var10 < var9; ++var10) {
                String s = var8[var10];
                this.searchForWhereOne(searchType, orderLambdaQueryWrapper, s);
            }
        } else {
            this.searchForWhereOne(searchType, orderLambdaQueryWrapper, searchContent);
        }

        Page<Order> orderPage = new Page<>((long)page, (long)num);
        Page<Order> orderPage1 = this.page(orderPage, orderLambdaQueryWrapper);
        JSONObject jsonObject = new JSONObject();
        jsonObject.fluentPut("info", orderPage1.getRecords()).fluentPut("total", orderPage1.getTotal());
        return R.success(jsonObject);
    }

    private void searchForWhereOne(SearchType searchType, LambdaQueryWrapper<Order> orderLambdaQueryWrapper, String searchContent) {
        switch (searchType) {
            case WANG_ID -> orderLambdaQueryWrapper.like(Order::getWangNo, searchContent).or();
            case ALIPAY_ID -> orderLambdaQueryWrapper.like(Order::getAlipayNo, searchContent).or();
            case WECHAT_ID -> orderLambdaQueryWrapper.like(Order::getWechatNo, searchContent).or();
            case PHONE_NUMBER -> orderLambdaQueryWrapper.like(Order::getPhoneNumber, searchContent).or();
        }

    }

    public R<String> addOrderFlag(Integer orderId, Integer flagId, String flagRemark) {
        Order order = new Order();
        order.setId(orderId);
        order.setFlag(flagId);
        order.setFlagRemark(flagRemark);
        boolean update = this.updateById(order);
        return update ? R.success("订单插旗成功," + flagId + "号旗") : R.failed(HttpCodes.HTTP_CODES555, "订单插旗失败,请联系服务器管理员");
    }

    public R<String> changeOrder(Order order) {
        boolean update = this.updateById(order);
        return update ? R.success("订单修改成功") : R.failed(HttpCodes.HTTP_CODES555, "订单修改失败,请联系服务器管理员");
    }

    public R<String> delOrder(Integer orderId) {
        boolean delete = this.removeById(orderId);
        return delete ? R.success("删除订单成功") : R.failed(HttpCodes.HTTP_CODES555, "删除订单失败,请联系服务器管理员");
    }

    public R<String> confirm(OrderStatus orderStatus, Integer orderId) {
        Order order = new Order();
        order.setId(orderId);
        order.setOrderStatus(orderStatus.name());
        boolean update = this.updateById(order);
        return update ? R.success("订单确认成功,当前状态:" + orderStatus.name()) : R.failed(HttpCodes.HTTP_CODES555, "订单确认失败,请联系服务器管理员");
    }

    @SneakyThrows
    public void downloadTemplate(HttpServletResponse response) {
        try {
            File file = new File("./template/example.xlsx");
            FileInputStream fileInputStream = new FileInputStream(file);
            BufferedInputStream bufferedInputStream = new BufferedInputStream(fileInputStream);
            byte[] bytes = new byte[bufferedInputStream.available()];
            bufferedInputStream.read(bytes);
            bufferedInputStream.close();
            response.reset();
            response.setCharacterEncoding("utf-8");
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            response.addHeader("Content-disposition", "attachment;filename*=utf-8''example.xlsx");
            response.addHeader("Content-Length", "" + file.length());
            BufferedOutputStream bufferedOutputStream = new BufferedOutputStream(response.getOutputStream());
            response.setContentType("application/octet-stream");
            bufferedOutputStream.write(bytes);
            bufferedOutputStream.flush();
        } catch (Throwable var7) {
            throw var7;
        }
    }

    @SneakyThrows
    @Transactional
    public R<JSONObject> uploadBatch(MultipartFile file) {
        DateFormat fmt =new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        AtomicInteger total = new AtomicInteger();
        AtomicBoolean status = new AtomicBoolean(false);
        EasyExcel.read(file.getInputStream(),OrderByExcel.class,new PageReadListener<OrderByExcel>(list -> {
            list.remove(0);
            total.addAndGet(list.size());
            List<Order> orders = new ArrayList<>();
            list.forEach(e -> {
                try {
                    orders.add(new Order()
                            .setOrderNo(e.getOrderNo())
                            .setOrderTime(e.getOrderTime() != null ? fmt.parse(e.getOrderTime()) : null)
                            .setOrderStatus(e.getOrderStatus())
                            .setWangNo(e.getWangNo())
                            .setWechatNo(e.getWechatNo())
                            .setAlipayNo(e.getAlipayNo())
                            .setPayAmt(e.getPayAmt() != null ? Double.parseDouble(e.getPayAmt()) : null)
                            .setStoreId(e.getStoreId() != null ? Integer.parseInt(e.getStoreId()) : null)
                            .setCommission(e.getCommission() != null ? Double.parseDouble(e.getCommission()) : null)
                            .setAlipayName(e.getAlipayName())
                            .setRpName(e.getRpName())
                            .setRpWechatName(e.getRpWechatName())
                            .setRemark(e.getRemark())
                            .setPhoneNumber(e.getPhoneNumber())
                            .setCardNo(e.getCardNo())
                            .setFlag(e.getFlag() != null ? Integer.parseInt(e.getFlag()) : null)
                            .setFlagRemark(e.getFlagRemark())
                            .setPayTime(e.getPayTime() != null ? fmt.parse(e.getPayTime()) : null)
                            .setAddress(e.getAddress())
                            .setShopId(e.getShopId() != null ? Integer.parseInt(e.getStoreId()) : null)
                            .setSku(e.getSku())
                    );
                } catch (ParseException ex) {
                    throw new RuntimeException(ex);
                }
                log.info(" ==> 解析一条数据: {}", JSONObject.toJSONString(e));
            });
            log.info(" ==> 共全部数据: {}",total.get());
            status.set(this.saveBatch(orders));
        })).sheet().doRead();
        return R.success(new JSONObject().fluentPut("total",total.get()).fluentPut("status",status.get()));
    }
}
