package top.xinsin.util;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.excel.exception.ExcelDataConvertException;
import com.alibaba.excel.read.processor.AnalysisEventProcessor;
import com.alibaba.excel.util.ListUtils;
import com.alibaba.fastjson2.JSONObject;
import lombok.extern.slf4j.Slf4j;
import top.xinsin.mapper.OrderMapper;
import top.xinsin.pojo.Order;
import top.xinsin.pojo.OrderByExcel;
import top.xinsin.service.OrderService;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author xinsin
 * Created On 2023/4/22 22:44
 * @version 1.0
 */
@Slf4j
public class ExcelParserUtil extends AnalysisEventListener<OrderByExcel> {
    /**
     * 每次读取的最大数据条数
     */
    private static final int MAX_BATCH_COUNT = 100;

    /**
     * 泛型bean属性
     */
    private OrderService dynamicService;

    /**
     * 可接收任何参数的泛型List集合
     */
    List<OrderByExcel> list = ListUtils.newArrayListWithExpectedSize(MAX_BATCH_COUNT);

    List<Order> total = new ArrayList<>();


    /**
     * 构造函数注入bean(根据传入的bean动态注入)
     *
     * @param dynamicService
     */
    public ExcelParserUtil(OrderService dynamicService) {
        this.dynamicService = dynamicService;
    }

    @Override
    public void invoke(OrderByExcel data, AnalysisContext analysisContext) {
        log.info(" ==> 解析一条数据: {}", data);
        list.add(data);
        if (list.size() > MAX_BATCH_COUNT) {
            // 保存数据
            saveData();
            // 清空list
            list.clear();
        }
    }
    /**
     * 所有数据解析完成后,会来调用一次
     * 作用: 避免最后集合中小于 MAX_BATCH_COUNT 条的数据没有被保存
     *
     * @param context
     */
    @Override
    public void doAfterAllAnalysed(AnalysisContext context) {
        saveData();
        log.info(" ==> 数据解析完成 <==");
    }
    /**
     * 保存数据
     */
    private void saveData() {
        log.info(" ==> 数据保存开始: {}", list.size());
        list.forEach(e -> {

            System.out.println(e.getOrderNo());
        });
//        dynamicService.saveBatch(list);
        log.info(" ==> 数据保存结束 <==");
    }

    /**
     * 在转换异常 获取其他异常下会调用本接口。我们如果捕捉并手动抛出异常则停止读取。如果这里不抛出异常则 继续读取下一行。
     *
     * @param exception
     * @param context
     * @throws Exception
     */
    @Override
    public void onException(Exception exception, AnalysisContext context) throws Exception {
        log.error(" ==> 数据解析失败,但是继续读取下一行:{}", exception.getMessage());
        //  如果是某一个单元格的转换异常 能获取到具体行号
        if (exception instanceof ExcelDataConvertException) {
            ExcelDataConvertException convertException = (ExcelDataConvertException) exception;
            log.error("第{}行，第{}列数据解析异常", convertException.getRowIndex(), convertException.getColumnIndex());
        }
    }
}
