package top.xinsin.util;

import com.auth0.jwt.JWT;
import com.auth0.jwt.JWTCreator;
import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.interfaces.DecodedJWT;

import java.util.Calendar;
import java.util.Map;

/**
 * @author xinsin
 * Created On 2023/4/3 15:47
 * @version 1.0
 */
public class JWTUtils {
    private static final String KEY = "a3Q%b5R@j7kP*9d-v'Y|h-%i(h9J0sU1/o";

    /**
     * 创建token
     * @param map 字段
     * @return token
     */
    public static String getToken(Map<String,String> map){
        Calendar instance = Calendar.getInstance();
        //默认两个小时过期
        instance.add(Calendar.HOUR,2);
        //创建jwt builder
        JWTCreator.Builder builder = JWT.create();
        //payload
        map.forEach(builder::withClaim);
        //指定令牌过期时间
        return builder.withExpiresAt(instance.getTime())
                .sign(Algorithm.HMAC512(KEY));
    }

    /**
     * 验证token是否合法
     * @param token token
     */
    public static void verify(String token){
        JWT.require(Algorithm.HMAC512(KEY)).build().verify(token);
    }

    /**
     * 从token中提取信息
     *
     * @param token token
     * @return token中包含的信息
     */
    public static DecodedJWT getTokenInfo(String token) {
        return JWT.require(Algorithm.HMAC512(KEY)).build().verify(token);
    }

    /**
     * 判断token与用户名是否对应
     *
     * @param token    token
     * @param username 用户名
     * @return 是否对应
     */
    public static boolean isUser(String token, String username) {
        String tokenUsername = getTokenInfo(token).getClaim("username").asString();
        return tokenUsername.equals(username);
    }
    public static Integer getId(String token){
        return Integer.valueOf(getTokenInfo(token).getClaim("id").asString());
    }
    public static Integer getAuth(String token) {
        return getTokenInfo(token).getClaim("auth").asInt();
    }
}
